// PointSprite with Vertex Col
// catweasel
//
float spritesize <string uiname="Sprite Size";> = 0.01f;

float4 spritecolor <string uiname="Sprite Color"; bool color=true;>;

Texture2D tex <string uiname="Texture";>;


//Camera transforms
float4x4 tVP : VIEWPROJECTION;
float4x4 tIV : VIEWINVERSE;
float4x4 tW : WORLD ;
float4x4 tWV : WORLDVIEW ;
float4x4 tV : VIEW ;
float4x4 tWVP : WORLDVIEWPROJECTION ;

struct vsIn
{
    float4 pos : POSITION;
	float4 Vcol : Color ;  
	uint vid : SV_VertexID ;
};

struct gsIn
{
    float4 pos : POSITION;
	float4 Vcol : Color ;
};

struct psIn
{
    float4 pos : SV_Position;
    float2 uv : TEXCOORD0;
	float4 Vcol : Color ;
};

//Quad position/uvs
float3 g_positions[4]: IMMUTABLE =
{
float3( -1, 1, 0 ),
float3( 1, 1, 0 ),
float3( -1, -1, 0 ),
float3( 1, -1, 0 ),
};

float2 g_texcoords[4]: IMMUTABLE = 
{ 
float2(0,1), 
float2(1,1),
float2(0,0),
float2(1,0),
};

SamplerState g_samLinear : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

gsIn VS(vsIn input)
{
	//Here we just pass trough, gs, with expand the point
    gsIn output;
	output.pos = mul( input.pos,tW);
	output.Vcol = input.Vcol; ///Just pass Vertext colors through
    return output;
}

[maxvertexcount(4)]
void GS(point gsIn input[1], inout TriangleStream<psIn> SpriteStream)
{
    psIn output;
	    	//Get position from quad array
    	float4 vpos = mul( float4(input[0].pos.xyz,1), (float4x4)tVP ) ;
    // create the quad	
    for(int i=0; i<4; i++)
    {
       float3 position = g_positions[i]*spritesize ;
    	
    	//Make sure quad will face camera
        position = mul( float4(position,0), tIV ).xyz + input[0].pos.xyz;
    	// Color Vertex
    	output.Vcol = input[0].Vcol;
    	//Project vertex
        output.pos = mul( float4(position,1.0), tVP );
    	//Assign UV's
        output.uv = g_texcoords[i];
        SpriteStream.Append(output);
    }
    SpriteStream.RestartStrip();
}

float4 PS(psIn input) : SV_Target
{   float4 c= tex.Sample( g_samLinear, input.uv ) * spritecolor * input.Vcol;;
	if(((c.r+c.g+c.b)/3)<0.02){discard;} // get rid of any nearly black pixels as alpha doesnt work in sprites well
	c.rgb = c.rgb;
    return c;
}


technique10 RenderParticles
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( CompileShader( gs_4_0, GS() ) );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );     
    }  
}


